/**
 * @fileOverview webpack开发配置
 * @author langford@126.com
 */

// process.traceDeprecation = true

const webpack = require("webpack");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const config = require("./webpack.base.config")({
  dev: true,
  env: "development",
});
const utils = require("../utils");
const abcConfig = utils.abcConfig;
const devConfig = abcConfig.env.development;
const FriendlyErrorsWebpackPlugin = require("friendly-errors-webpack-plugin");
const _ = require("lodash");

// 合并 development rules
const rules = [
  [
    {
      test: /\.css$/,
      use: [
        "style",
        "css",
        {
          loader: "postcss",
          options: utils.getPostCssOptions(),
        },
      ],
    },
    {
      test: /\.less$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            localIdentName: abcConfig.cssModule.localIdentName,
            importLoaders: 2,
            sourcemap: true,
          },
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions(),
        },
        {
          loader: "less",
          options: {
            javascriptEnabled: true,
          },
        },
      ],
    },
    {
      test: /\.s(c|a)ss$/,
      use: [
        "style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            localIdentName: abcConfig.cssModule.localIdentName,
          },
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions(),
        },
        "sass",
      ],
    },
    {
      test: /\.styl(us)?$/,
      use: [
        "vue-style",
        {
          loader: "css",
          options: {
            modules: abcConfig.cssModule.enable,
            minimize: true,
            localIdentName: abcConfig.cssModule.localIdentName,
          },
        },
        {
          loader: "postcss",
          options: utils.getPostCssOptions(),
        },
        "stylus",
      ],
    },
  ],
  config.module.rules,
  devConfig.rules,
];
config.module.rules = utils.merge(rules, "rule");

// 合并 development plugins
const plugins = [
  [
    new webpack.DefinePlugin({
      __DEV__: true,
      NODE_ENV: JSON.stringify("development"),
    }),
    new FriendlyErrorsWebpackPlugin(),
    new webpack.HotModuleReplacementPlugin(),
  ].concat(
    abcConfig.enableHtmlWebpackPlugin
      ? [
          new HtmlWebpackPlugin({
            template: devConfig.htmlTemplateUrl || abcConfig.htmlTemplateUrl,
            inject: "body",
            ...abcConfig.htmlWebpackPluginOptions,
            ...devConfig.htmlWebpackPluginOptions,
          }),
        ]
      : []
  ),
  config.plugins,
  devConfig.plugins,
];

config.externals = _.merge(
  {},
  config.externals || {},
  abcConfig.externals || {}
);

config.plugins = utils.merge(plugins, "plugin");
config.devServer = {
  ...abcConfig.devServer,
  open: abcConfig.open || true,
  hot: abcConfig.hot || true,
  historyApiFallback: abcConfig.historyApiFallback || true,
  inline: abcConfig.inline || true,
  host: abcConfig.host,
  port: abcConfig.port,
  proxy: utils.getProxyConfig(),
  disableHostCheck: abcConfig.disableHostCheck || true,
  // quiet: true,
  publicPath: abcConfig.output.publicPath || "/",
  contentBase: abcConfig.staticPath || process.cwd(),
  https: abcConfig.https || false,
  headers: abcConfig.headers || { "Access-Control-Allow-Origin": "*" },
};

config.mode = "development";

module.exports = config;
